#' Makes a doge meme from word frequency matrix.
#' @param textfile   A csv of text to plot.
#' @param numberOfWords Number of top terms from word frequency matrix to plot.
#' @examples 
#' #textfile <- read.csv("mytextfile.csv",stringsAsFactors=FALSE)
#' #numberOfWords <- 8
#' #make.doge(textfile,numberOfWords)
#' @references
#' [1] Doge, Know Your Meme - http://knowyourmeme.com/memes/doge
#' @export


make.doge <- function(textfile,numberOfWords){
  suppressPackageStartupMessages(if(!require(jpeg, quietly = TRUE)){
    install.packages("jpeg", quiet = TRUE, dependencies = TRUE)
    library(jpeg, quietly = TRUE)})
  
  suppressPackageStartupMessages(if(!require(lattice, quietly = TRUE)){
    install.packages("lattice", quiet = TRUE, dependencies = TRUE)
    library(lattice, quietly = TRUE)})
  
  suppressPackageStartupMessages(if(!require(grid, quietly = TRUE)){
    install.packages("grid", quiet = TRUE, dependencies = TRUE)
    library(grid, quietly = TRUE)})
  
  suppressPackageStartupMessages(if(!require(tm, quietly = TRUE)){
    install.packages("tm", quiet = TRUE, dependencies = TRUE)
    library(tm, quietly = TRUE)})  
  library(jpeg)
  #library(utils)
  library(lattice)
  library(grid)
  library(tm)
  
  review.source <- VectorSource(textfile)
  corpus <- Corpus(review.source)
  
  ## cleaning
  corpus <- tm_map(corpus, content_transformer(tolower))
  corpus <- tm_map(corpus, removePunctuation)
  corpus <- tm_map(corpus, stripWhitespace)
  corpus <- tm_map(corpus, removeWords, stopwords("english"))
  
  dtm <- as.matrix(DocumentTermMatrix(corpus))
  freq <- colSums(dtm)
  topwords <- freq[order(freq,decreasing=T)][1:numberOfWords]
  words <- names(topwords)
  words <- words[sample(c(1:numberOfWords),numberOfWords)]

  quals <- c("very","much","so","many")
  to.plot <- c(c("wow","WOW"),paste(quals,words))
  
  x.coord <- seq(-.2,.8,length.out=20)
  y.coord <- seq(0,1,length.out=20)
  long.word <- which(nchar(to.plot)>5)
  
  
  cols <- c("red","green","blue","purple","orange")
  myurl <- "http://i1.kym-cdn.com/photos/images/newsfeed/000/581/296/c09.jpg"
  z <- tempfile()
  download.file(myurl,z,mode="wb",quiet=T)
  
  windowsFonts(A=windowsFont("Comic Sans MS"))
  
  img <- as.raster(readJPEG(z))  
  plot.new()
  par(xpd=NA)
  grid.raster(img)
#  abline(v=seq(-2,2,.1),h=seq(-2,2,.1))
#  abline(v=0,h=0,lwd=2)
  xplot <- sample(x.coord,length(to.plot),replace=F)
  yplot <- sample(y.coord,length(to.plot),replace=F)
  no.go.zone <- xplot>=.2 & xplot<=.7 & yplot>=.4 & yplot<=.8
  nope <- sum(no.go.zone==T)
  while(nope>0){
      xplot[which(no.go.zone==T)] <- sample(x.coord,nope)
      yplot[which(no.go.zone==T)] <- sample(y.coord,nope)
      xplot[long.word] <- sample(x.coord[which(x.coord<.6)],length(long.word))
      no.go.zone <- xplot>=.3 & xplot<=.6 & yplot>=.5 & yplot<=.85
      nope <- sum(no.go.zone==T)
  }
  pairs <- cbind(xplot,yplot)
  text(xplot,
       yplot,
       to.plot,cex=1.8,col=cols,font=2,pos=4,family="A")
  invisible(file.remove(z))
}
